;********************************************************************
; Monitor fr CP/M-Moppel
;
; mrt55-18-122.asm		Monitor rot V55 Stand 2018 V1.22
;
;	
; Stand:
;	16.02.2018 - EZU Berichtigt
;	29.01.2018 - Quellcode auf Include-Dateien verteilt
;	28.01.2018 - EZU von Multi-IO-Karte eingebaut
;	27.01.2018 - Fehlerbereinigung OK
;	26.01.2017 - HEX-Teile entfernen
;			 Multi-IO, CP/M-Bootloader	Adr.: 0003h
;			 Intel-Hex-lader eingebaut	Adr.: 0040h
;                    
;--------------------------------------------------------------------
;
#include	"sys-18-1.asm"	; Belegung Systemram
;
#include	"spv-18-1.asm"	; Sprungtabelle von V-Monitor V7.5
;
#include	"io-18-1.asm"	; IO-Adressen
;
;
;********************************************************************
;
		.org	0000H		; ROM-Platz #0 auf CPU-Karte
	
rst0:		jmp	kaltst	; RST0	--> Kaltstart
cpm:		jmp	cpmb		; CP/M boot
fquarz:	.db  	06h		; Taktfrequenz in MHz 
timprl:	.db  	04h		; Prellzeit der Tastatur in ms
rst1:  	jmp 	rst1_m	; RST1
dely1:	jmp 	dely1_m	; DELY1 --> 1ms Laufzeit
prtbdr:	.db  	00h		; Baudrate fuer Drucker
		.db  	48h		; 4800 Bd
rst2:		jmp 	rst2_m	; RST2
dely100: 	jmp 	dely100_m	; DELY100 --> mit SSG-Anzeige
casbdr:	.db  	00h		; Baudrate fuer Cassetten
		.db  	12h		; 1200Bd
rst3:		jmp 	rst3_r	; RST3
onesec: 	jmp 	onesec_m	; ONESEC --> 1s Laufzeit
v24bdr:	.db  	00h		; Baudrate fuer V24-Schnittstelle
		.db  	48h		; 4800Bd
rst4:		jmp 	rst4_r	; RST4
		.db  	00h		; 
trap:		pchl			; TRAP	--> FDC-Routinen
		jmp 	trap_r	;
rst5:		jmp 	rst5_r	; RST5  
		nop			;
rst55:	jmp 	rst55_r	; RST5.5 --> FDC-Routinen
		nop			;
rst6:		jmp 	rst6_r	; RST6
		nop			;
rst65:	jmp 	rst65_r	; RST6.5 
		nop			;
rst7:		jmp 	rst7_r	; RST7
		nop		;
rst75:	jmp 	rst75_r	; RST7.5 
		.db  	55h		; Monitorversion 5.5
ihex:		jmp 	i_hex		; Intel-Hex lader
ci:		jmp 	ci_v		; CI ASCII-Tastatur abfragen (A)
ri:		jmp 	ri_m		; RI ein Byte von Kasetteninterface empfangen (A)
co:		jmp 	co_m		; CO Inhalt(C) ASCII an Bildschirm (UPDATE)
po:		jmp 	po_m		; PO Inalt (C) auf Kasetteninterface (1200Bd)
lo:		jmp 	lo_m		; LO Inalt (C) ASCII-Zeichen an serielle Scnitstelle (4800Bd)
csts:  	jmp 	csts_v	; CSTS Status ASCII-Tastatur gedrueckt Z=0 sonst Z=1
csinit:	jmp 	csinit_m	; 
loinit:	jmp 	lonit_m	; LO-Init
maxram:	jmp 	maxram_m	; MEMCHK Speichertest (AB)max Adressen
prtmod:	.db  	15h		; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)
casmod:	.db  	10h		; Cassettenbetriebsart (8daten, ohne Paritaet)
v24mod:	.db  	15h		; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)
time:		jmp 	time_m	; TI Uhrzeit lesen ASCII in Buffer (HL)String
v24ini:	.dw  	v24ini_m	; wird ueber PCHL aus Videomonitor angesprungen
		.db  	0e9h		; = PCHL
cnvbyt: 	jmp 	0000h		; cnvbyt_m	; SSG-Umsetzen
reles:	jmp 	0000h		; reles_m	; HEX-Tastatur auf Loslassen warten
press:  	jmp 	0000h		; press_m	;  "           auf Drcken warten
hhkey:	jmp 	0000h		; hhkey_m	; HEX-Input
cckey:  	jmp 	0000h		; cckey_m	; CMD-Input
datset: 	jmp 	0000h		; datset_m	; Daten-Mode einstellen
writur:	jmp 	wr_uhr	; Uhrzeit stellen (SRCBEG)Std,min, 
					; (SRCEND)Wochentag, (DTSBEG)Monat,Jahr
readur:	jmp 	rd_uhr	; READUR Uhrzeit lesen BCD(HL)Adresse 
save:   	jmp 	save_m	; Register vorbereiten/sichern 
restor:	jmp 	restor_m	; Registersatz laden, Start
fill:		jmp 	fill_m	; FILL Speicher fuellen (SRCBEG,SRCEND,DSTBEG)
copy: 	jmp 	copy_m	; COPY Speicher kopieren (SRCBEG,SRCEND,DSTBEG)
casot:	jmp 	casot_m	; Casetten Ausgabe
casin:	jmp 	casin_m	; Eingabe
exchan:	jmp 	exchan_m	; Copy mit Adressumsetzung
ezudsp:	jmp 	0000h		; ezudsp_m	; Uhr in SSG-Anzeige
dspacc:	jmp 	0000h		; dspacc_m	; (ACC) darstellen
dsphl:	jmp 	0000h		; dsphl_m	; (HL) darstellen
dsperr: 	jmp 	0000h		; dsperr_m	; Fehleranzeige
;
;
kaltst:	lxi	sp,stack	;
		call	save		; Register vorbereiten/sichern
		call	init		; Init
		lxi	h,keybyt	; Key true oder invers 
		mvi	m,0ffh	;     
		jmp	kalt_v	; zum Videomonitor
;	
;
; INIT 	nimmt die Grundeinstellung vor
;	
init:  	lxi	h,usrstk	; Anwenderstack
		mov	a,l		; sortieren fuer REGBUF
		mov	l,h		; H&L tauschen
		mov	h,a		;
		shld	reg_s		; Stackpointer retten
		mvi	a,00h		;
		sta	0000h		; ! bringt imEPROM nichts
		mvi	a,00h		;
		sta	modflg	; Flags loeschen
		sta	ioflg		; IO-Byte
		call	adrfix	;
		call	dat		;
		mvi	a,00h		;
		sta	regnr		; REG A voreinstellen
		out	40h		; FDC Selektbyte
		out	prom_ctl	; Promer
		mvi	a,0ch		;
		out	therm_d	; Thermodrucker
		mvi	a,20h		;
		out	cs_mode	; Serielle Schnittstelle
		mvi	a,82h		;
		out	par_ctl	; Parallele Schnittstelle
		mvi	a,0f0h	;
		out	prom_adrh	; Promer LEDausschalten
		mvi	a,0f4h	;
		out	uhr_ctl	; Ruhepegel fuer Uhr
		mvi	a,17h		; RSTs sperren
		call	cfinit	; CF-Karte
		call	mioini	; Multi-IO-Karte
		sim			; Interupptmaske
		nop			; wg. NSC800 JR-Befehl
		in	fdc_dat	;
		out	cs_bell	; Summer
		ret			;
;
; SAVE 		rettet alle Register
;
save_m:	sta	reg_a		; Register sichern/vorbereiten 
		mov	a,l		; H mit L tauschen
		mov	l,h
		mov	h,a
		shld	reg_h		; HL sichern 
		push	psw
		pop	h		; PSW in HL
		mov	a,l
		sta	reg_f		; PSW sichern
		lxi	h,reg_b	; Zeiger auf REG-Buffer
		mov	m,b		; REG B 
		inx	h
		mov	m,c		; REG C
		inx	h
		mov	m,d		; REG D
		inx	h
		mov	m,e		; REG E
		lxi	h,0000h	; SP vorbereiten
		dad	sp
		inx	h
		inx	h
		mov	a,l		; (H&L) tauschen
		mov	l,h
		mov	h,a
		shld	reg_s		; SP sichern
		ret			;
;
;
; RUN	Programmstart bei der im Adressfeld stehende Adresse
;	laed vorher alle Register inkl.Flags
;
restor_m:	lda modflg
		ani	80h		; STP-Mode ?
		cnz	rst1		; ja: Breakpoint setzten
		lda	reg_f		; Flags holen
		mov	c,a
		push	b		; aufStack sichern
		pop	psw		; und ins CPU-Flag-Register
		lhld	reg_s		; Anwenderstack holen
		mov	a,l		; sortieren
		mov	l,h
		mov	h,a
		sphl			; in CPU-STACK
		lxi	h,reg_b	; REB B laden
		mov	b,m		;
		inx	h		;
		mov	c,m		; REG C laden
		inx	h		;
		mov	d,m		; REG D laden
		inx	h		;
		mov	e,m		; REG E laden
		lhld	adres		; Startadresse holen
		push	h		; im Stack sichern
		lhld	reg_h		; REG H laden
		mov	a,l		; sortieren
		mov	l,h		;
		mov	h,a		;
		lda	reg_a		; Reg A laden
		ret			; Programmstart
;
; Mode-Keys DAT, ADR/FCTF, REG, FCT
; Hex-Teile gelscht
;
dat:  	lda	modflg
		ani	0fdh		; ADR-Flag loeschen
		ori	01h		; Dat-Flag setzen
		jmp	modcte
		;
adrfix: 	lxi	h,rambeg	; Anfangsadresse RAM
adrse:  	shld	adres		; ins Adressfeld laden
		lda	modflg
		ani	0f2h		; DAT,REG,FCT-Flag loeschen
		ori	02h		; ADR-Flag setzen
		;
modcte: 	sta	modflg
		ret
;		
;====================================================================
;
; Laufzeitprogramme
;
;	
onesec_m:	push	psw		; 1s warten
		mvi	a,0ah		; 10d
onelop: 	call	dely100_m	;	
		dcr	a
		jnz	onelop
		pop	psw
		ret
;
dely100_m:	push	psw		; 100ms warten
		mvi	a,0ah
delylp: 	dcr	a
		jnz	delylp
		pop	psw
		ret
;
dely7m: 	push	psw		; Tastaturprellzeit
		lda	timprl
dlop:   	call	dely1		; 7 ms warten
		dcr	a
		jnz	dlop
		pop	psw
		ret	
;	
dely2m:	call	dely1		; 2 ms warten
dely1_m: 	push	psw		; 1ms warten
		lda	fquarz
		add	a
		add	a
		inr	a
		add	a
		add	a
		add	a
delop:  	dcr	a
		jnz	delop
		pop	psw
		ret		
;
;********************************************************************
; 
; STP/RAM Einzelschritt und RAM-Funktion
;
; Drei Betriebsarten mglich:
;
;	1. Einzel-Byte-Betrieb
;	   Mit Einzelschrittmodul; RAM oder ROM
;
;	2. Breakpointbetrieb
;	   Softwarehaltepunkt im RAM setzen
;
;	3. Einzelschritt-Betrieb
;	   Single-Step im RAM
;
;
;	.org 0500h
; Einzelschritt-Betrieb im RAM
;
;
; REST1	ersetzt den naechsten Opcode
;		durch einen Breakpoint (RST2 = D7h)
;
rst1_m:	lhld	adres		; aktuelle Adresse holen
		call	rest11	; Adr. des naechsten OP-Code
		mov	a,m
		sta	opcod1	; und retten
		mvi	m,0d7h	; "RST2" einsetzen
;
; REST12	setzt bei den Befehlen 
;		PCHL, RST, RET, JMP bzw CALL einen zweiten
;		Breakpoint an der moeglichen Zieladresse ein
;		RST2 = D7h
;
rest12:	ldax	d		; Opcode erneut holen
		cpi	0e9h		; PCHL ?
		jnz	rstct2	; nein weiter
		lhld	reg_h		; Ziel in (HL)
		mov	a,l		; sortieren
		mov	l,h		;
		mov	h,a		;
		jmp	rstend	;
					;
rstct2: 	call	rstbef	; ist es ein RSTn ?
		jz	rstct3	; nein, weiter
		ldax	d		;
		ani	38h		; Zieladressbit maskieren
		inr	a		; Adresse hinter "C3"
		mov	l,a		;
		mvi	h,00h		; (HL) 01...39h
		mov	e,m		; Zieladresse low
		inx	h
		mov	d,m		; Zieladresse high
		xchg			; (HL) RST-Sprungziel
		jmp	rstend	;
					;
rstct3: 	call	retbef	; ist es ein RET ?
		jz	rstct4	; nein, weiter
		lhld	reg_s		; Userstack-Pointer
		mov	a,l		; sortieren
		mov	l,h
		mov	h,a
		mov	e,m		; Stack auslesen
		inx	h
		mov	d,m		; (DE) RET-Adress
		xchg
		jmp	rstend	;
					;
rstct4: 	call	jmpbef	; ist es ein JMP/CALL ?
		rz			; nein, ende
		inx	d		; Zieladresse holen
		ldax	d
		mov	l,a		; untere Adresshaelfte
		inx	d
		ldax	d
		mov	h,a		; obere Adresshaelfte
					;
rstend: 	shld	brkpt2	; Break-Adresse 2 retten
		mov	a,m		; Opcode holen
		sta	opcod2	;
		mvi	m,0d7h	; "RST2" einsetzen
		ret
;
; REST11	ermittelt die Adressen des naechsten Opcode
;		(HL) Adresse des aktuellenOpcode
;
rest11: 	push	h		;
		xchg			; (DE) Adr. aktuellen Opcode
		mvi	b,03h		; Schrittweite fuer Dreiwort-Befehle
		call	drwbef	; ist es ein Dreiwort Befehl ?
		jnz	rstcte	; ja, weiter
		dcr	b		; Schrittweite Zweiwort-Befehl
		call	zwwbef	; ist es ein Zweiwort Befehl ?
		jnz	rstcte	; ja, weiter
		dcr	b		; Schrittweite Einwort-Befehl
rstcte: 	mov	c,b		;
		mvi	b,00h		; (BC) 1,2,3
		pop	h
		dad	b		; (HL) naechster Opcode
		shld	brkpt1	; sichern
		ret
;
zwwbef: 	lxi	h,op2tab	; Tabelle Zweiwort-Befehleg
		mvi	c,12h		; Tabellenlaenge 18d
		jmp	oplop		;
		;
drwbef: 	lxi	h,op3tab	; Tabelle Dreiwort-Befehle
		mvi	c,1ah		; Laenge 26d (Berichtigt aus HAT1)
		jmp	oplop		;
		;
jmpbef: 	lxi	h,jmptab	; Tabelle JMP/CALL-Befehl
		mvi	c,12h		; Laenge 18d
		jmp	oplop		;
					;
retbef: 	lxi	h,retab	; Tabelle Return-Befehl
		mvi	c,09h		; Laenge 9d
		jmp	oplop
		
rstbef: 	lxi	h,rstab	;  Tabelle Restart-Befehl
		mvi	c,08h		; Laenge 8
oplop:  	ldax	d		; Opcode holen
		cmp	m		; REG A = Tabwert
		jz	opend		; ja, weiter
		inx	h
		dcr	c
		jnz	oplop		; Schleife 
opend:  	xra	a		; REG A loeschen
		cmp	c		; Tab durchlaufen
		ret			; ja, (Z)=1 ende
;
; REST2	ersetzt Breakpoint durch den 
;		urspruenglichen Opcode
; 
rst2_m: 	shld	hibuf		; HL retten
		sta	hbuf		; REG A retten
		pop	h		; gleicht RST-Interupt aus
		dcx	h		; wg Einwort Interupt
		shld	adres		; aktualisierte Adresse
		lhld	brkpt1	; Breakadresse 1 holen
		lda	opcod1	; und geretteten Opcode 1
		mov	m,a		; wieder einsetzen
		lhld	brkpt2	; Breakadresse 2 holen
		lda	opcod2	; und geretteten Opcode 2
		mov	m,a		; wieder einsetzen
		lda	hbuf		; REG A laden
		lhld	hibuf	; und HL
		call	save		; saemtliche REGs in RAM-Buffer
		lxi	h,0000h	; Brakadresse loeschen
		shld	brkpt1
		shld	brkpt2
		push	h		; Eintragungen im User-Stack
		push	h		; loeschen
		pop	h
		pop	h
		lxi	sp,stack	; Monitor Stack
		jmp	hibuf2	; Warmstart
;
; Tabelle
;	
op2tab:	.db	06h		; Zweiwort-Befehle
		.db	0eh
		.db	16h		;
		.db	1eh	
		.db	26h		;
		.db	2eh	
		.db	36h		;
		.db	3eh
		.db	0c6h		;
		.db	0ceh		
		.db	0d6h		;
		.db	0deh	
		.db	0e6h		;
		.db	0eeh	
		.db	0f6h		;
		.db	0feh
		.db	0d3h		;
		.db	0dbh	
op3tab:	.db	01h		; Dreiwort-Befehle
		.db	11h		
		.db	21h		;
		.db	22h	
		.db	2ah		;
		.db	31h
		.db	32h		;
		.db	3ah
jmptab:	.db	0c2h		; JMP-Tab
		.db	0c3h	
		.db	0cah		;
		.db	0d2h	
		.db	0dah		;
		.db	0e2h		
		.db	0eah		;
		.db	0f2h
		.db	0fah		;
caltab:	.db	0c4h
		.db	0cch		;
		.db	0cdh	
		.db	0d4h		;
		.db	0dch
		.db	0e4h		;
		.db	0ech
		.db	0f4h		;
		.db	0fch
retab:	.db	0c0h		; 
		.db	0c8h
		.db	0c9h		;
		.db	0d0h		 
		.db	0d8h		; 
		.db	0e0h		 
		.db	0e8h		; 
		.db	0f0h		 
		.db	0f8h		;
rstab:	.db	0c7h		; 
		.db	0cfh		; 
		.db	0d7h		 
		.db	0dfh		; 
		.db	0e7h		 
		.db	0efh		; 
		.db	0f7h		 
		.db	0ffh		; 
;
;
;====================================================================
;
; RAM-Verwaltung
;
; EXCHAN	kopiert Speicherbereiche um und
;		passt dabei diejenigen Adressen
;		an, die im kopierten Block liegen
;
exchan_m:	call	copy		; umkopieren
		lhld	srcbeg	; Adressdifferenz bilden
		xchg	
		lhld	dstbeg	;
		shld	brkpt1	; Anfang Zielbereich
		mov	a,l
		sub	e
		mov	l,a		; Differenz low
		mov	a,h
		sbb	d
		mov	h,a		;Differenz high
		shld	dstend
		xchg
		lhld	srcend
		dad	d		; Ende Zielbereich
		shld	brkpt2	; fuer Endabfrage ablegen
exlop:  	lhld	brkpt1	; aktuellerOpcode
		shld	hibuf		; Adresse retten
		call	rest11	; naechster Opcode n.BRKPT1
		xchg			; (DE) Adr naechster Opcode)
		lhld	brkpt2	; (HL) Endadresse Zielbereich
		call	comp16	; Ende ?
		rnc			; ja,ende
		lhld	hibuf		; Adresse zurueckholen
		xchg
		call	drwbef	; ist ein Dreiwort-Befehl
		cnz	modify	; ja,weiter
		jmp	exlop		; Schleife
;
;
; CMPEND	vergleicht (HL) mit SRCEND
;		(HL) > SRCEND, dann Bereichsende
;		(CY) = 1
;	
cmpend: 	xchg			;(DE) RAM-Adresse
		lhld	srcend
comp16: 	mov	a,d		; 16Bit Vergleich
		cmp	h
		rc			; Adr. > SRCEND, ende
		mov	a,e
		cmp	l		; Adr. = SRCEND, (Z) = 1
		ret			; ADR. > SRCEND, (CY) = 1
;
modify: 	inx	d		; (DE) Adr des aktuellen Opcode
		ldax	d		; Absolute Adresse low
		mov	l,a
		inx	d
		ldax	d		; Absolute Adresse high
		mov	h,a		; (HL) Absolute Adresse
		call	cmpend	; Vergleich mit SRCEND
		rnc			; > als SRCEND,ende
		lhld	srcbeg	; Adressdifferenz
		call	comp16	; Vergleich mit SRCEBEG
		jz	mod		; 
		rc			; < als SRCBEG, ende
mod:  	lhld	dstend	; Adressdifferenz
		dad	d		; zur absoluten Adresse addieren
		xchg			; (DE) modifizierte Adresse
		lhld	hibuf		; aktueller Opcode
		inx	h		; zweites Byte im Befehl
		mov	m,e		; modifiz. Adresse low
		inx	h		; drittes Byte im Befehl
		mov	m,d		; modifiz. Adresse high
		ret			;
;
; INSERT	fuegt an der aktuellen RAM-Adresse
;		NOPs (=00) ein (RAM-Ende beachten)
;		keine Adressanpassung
;
fctins: 	lda	adres		; Anzahl holen
		push	psw
		mov	e,a
		mvi	d,00h
		lhld	srcbeg
		push	h
		dad	d		; Zieladresse
		shld	dstbeg
		lxi	h,buff_poi
		shld	srcend
		call	copy_m	; umkopieren
		pop	h
		pop	psw
		push	h
inslop: 	mvi	m,00h		; NOP einfuegen
		inx	h
		dcr	a
		jnz	inslop
		pop	h
		jmp	datset	; Daten-Mode
;
; DELETE	loescht RAM-Zellen und rueckt RAM-Inhalt nach
;		bis RAM-Ende, keine Adressanpassung
;
fctdel: 	lda	adres		; Anzahl holen
		mov	e,a
		mvi	d,00h
		lhld	srcbeg
		shld	dstbeg
		push	h
		dad	d		; Startadresse
		shld	srcbeg
		lxi	h,buff_poi
		shld	srcend
		call	copy_m	; umkopieren
		pop	h
		jmp	datset	; Daten-Mode
;

; RAM-Bereich verschieben
;	
;	
copy_m: 	lhld	srcend	; COPY Speicherbereich verschieben
		xchg			; (DE) Ende Quellbereich
		lhld	dstbeg	; (HL) Anfang Zielbereich
		call	maxcom	; Vergleich (DE) mit (HL)
		mov	b,h
		mov	c,l		; (BC) Anfang Zielbereich
		lhld	srcbeg	; (HL) Anfang Quellbereich
		jnc	copyup
		mov	d,b
		mov	e,c		; (DE) Anfang Zielbereich
		call	maxcom	; vergleich
		push	h
		lhld	srcend
		xchg			; (DE) Ende Quellbereich
		pop	h		; (HL) Anfang Quellbereich
		jnc	copyup
		push	d
		push	h
		mov	a,l		; DSTEND berechnen
		cma
		mov	l,a
		mov	a,h
		cma	
		mov	h,a
		xchg			; (DE) kompl. Anfang Quellbereich
		lhld	srcend
		dad	d
		xchg			; (DE) Differenz SRCEND - SRCBEG
		lhld	dstbeg
		dad	d
		inx	h		; (HL) Ende Zielbereich
		mov	b,h
		mov	c,l
		pop	d
		pop	h
		;
copydw: 	mov	a,m		; abwrts kopieren
		stax	b
		call	comp
		rz			; fertig, ende
		dcx	h
		dcx	b
		jmp	copydw
					;	
copyup: 	mov	a,m		; aufwrts kopieren
		stax	b
		call	comp
		rz			; fertig, ende
		inx	h
		inx	b
		jmp	copyup
					;	
comp:  	mov	a,l		; Vergleich
		cmp	e
		rnz			; ungleich, ende
		mov	a,h
		cmp	d		; gleich (Z)=1
		ret	
;
maxcom: 	mov	a,h		; Vergleich
		cmp	d
		rc			; (DE) > (HL), ende
		rnz
		mov	a,l
		cmp	e
		ret			; (DE) > (HL), (CY)=1
;
; RAM mit Konstanten fuellen
;
; FILL_m	fuellt RAM mit Konstante auf
;		SRCBEG = Anfangsadresse
;		SRCEND = Endadresse
;		DSTBEG = Konstante
;			
fill_m: 	lhld	srcend	; 
		xchg			; Anfang
		lhld	srcbeg	; Ende
		lda	dstbeg	; Konstante in DSTBEG
		mov	b,a		; Konstante
fillop: 	mov	m,b
		call	comp		; Ende ?
		inx	h
		rz
		jmp	fillop
;
; MAXRAM_m
;		ermittelt die hoechste RAM-Adresse (ab 8000h)
;		uebergabe in
;		(A) high, (B) low
;
maxram_m:	lxi	h,cpybuf	; Speichertest
		lxi	d,00fdh	; Grosse Speicherkarte
		mvi	b,49h		; Konstante
maxlop:	dad	d
		mov	c,m		; RAM-Inhalt retten
		mov	m,b		; Konstante in RAM
		mov	a,m		; Prueflesen
		mov	m,c		; alten Inhalt zurueckschreiben
		cmp	b		; Einschreiben erfolgreich ?
		jz	maxlop	; ja, weiter
		mov	a,h
		dcr	a		; MAX-Adr. high
		mvi	b,0ffh	; MAX-Adr. low
		ret			;
;

;
;********************************************************************
; 
; Ansteuerung von Peripheriegerten
;	
; Unterprogramme
;	- CSINIT_m	Initialisierung	CAS-Interface
;	- LOINIT_m		"	PRT-Interface
;	- V24ini_m		"	V24 Schnittstelle
;	- PO		Casettenausgabe
;	- RI		Casetteneingabe
;	- V24Out	Ausgabe V24
;	- V24in		Eingabe V24
;	- CPUOT		SOD-Ausgabe
;	- CPUIN		SID-Eingabe
;
;	
csinit_m:	push	d		; Casetteninterface vorbereiten
		push	h
		lda	casmod	; Modus laden
		mov	b,a
		mvi	a,0c2h	;
		lhld	casbdr	; Baudrate
		jmp	initct
					;
lonit_m:	push	d		; Serieller Drucker vorbereiten
		push 	h
		mvi	a,8ah		; 8255 initialisieren
		out	par_ctl	; PA, PClo = Out
		mvi	a,01h		; PB, PChi = IN
		out	par_pc	; Strobe auf High
		lxi	h,buff_txt	; Themodrucker vorbereiten
		shld	buff_poi	;
		xra	a		;
		sta	buff_num	; Zeichenzaehler loeschen
		mvi	a,01h		;
		out	0bah		; Centronis vorbereiten
		lda	prtmod	;
		mov	b,a		;
		mvi	a,0a0h	; Moderegister
		lhld	prtbdr	;
		jmp	initct	;
;
; Sprungtabelle fuer V24 Routinen
; werden ueber PCHL angesprungen
;
; 06.06.2017	Timer als BCD-Zaehler
;			ACIA Betriebsart auf Asynchron umgestellt
;	
v24ini_m:	jmp	initv24	; --> INIT V24
		jmp	lo7		; V24 Ausgabe
		jmp	ri_m		; V24 Eingabe
;	
initv24:	push	d		; INIT V24-Schnittstelle
		push	h
		lda	v24mod	;
		mov	b,a
		mvi	a,58h
		lhld	v24bdr	; Baudrate laden
initct:	out	cs_mode	; Modus ??
		mvi	a,03h
		out	acia_ctl	; Reset ACIA
		mov	a,b
		out	acia_ctl	; Datenuebertragung (bit,1Stop,ohne Paritaet)
		mvi	a,37h		; als BCD-Zaehler
		out	timer_ctl	; Timer_0 Mode 3, erst unt., dann ober.Byte
		call	bd_hol	; Bd-Werte ermitteln Rueckgabe (HL)
		mov	a,l		; Startwerte fuer Baudrate laden
		out	timer_0	; Teilfaktor low
		mov	a,h
		out	timer_0	; Teilfaktor high
		pop	h
		pop	d
		ret
;
; BD_HOL (BAUDRT)	
;		ermittelt aus der Baudrate und
;		CPU Takt (FQuarz) den Timer Teilfaktor
;		Siehe Tabelle
;					
bd_hol: 	mov	a,h		; Timerwerte fuer Baudrate setzen
		rlc
		mvi	a,00h
		adc	h
		ani	07h		; Versatz
		rlc	
		mov	e,a
		mvi	d,00h		; Zeiger innerhalb der Tabell
		lxi	h,bd_t	; Tabelle hohe Frequenz auswaehlen
bd_end:	dad	d
		mov	e,m		; Werte aus Tabelle holen
		inx	h
		mov	d,m
		xchg
		ret			; (HL)Timerwerte fuer Baudrate
;
; Baudratentabell als Binaerwert
;
; 06.06.2017 Tabelle fuer den Asynchronbetrieb neu aufgebaut
; (urspruenglich nur Synchronbetrieb)
;
; Teilfaktor fuer Timer = CPU-Frequenz /2 = Systemtakt / 4800bd / 16 = 40
;                                                                  I
;                                                                  UART Teiler
; Wert als BCD, da Timer auf BCD Zaehler eingestellt wird
;               sonst Umrechnung nach Binaer notwendig
;
; 9600Bd = 20h lowByte
; 4800Bd = 40h .
; 2400Bd = 80h .
; 1200Bd = 60h lowByte
;	   01h highByte
;
;
; Quarz > 5 MHz < 7MHz , hier 6,144MHz	
;
bd_t:		.db	40h		; 4800Bd
		.db	00h
		.db	45h		;  110Bd
		.db	17h
		.db	60h		; 1200Bd
		.db	01h
		.db	40h		;  300Bd
		.db	06h
		.db	80h		; 2400Bd
		.db	00h
		.db	40h		; 4800Bd
		.db	00h
		.db	20h		;  600Bd
		.db	03h
		.db	20h		; 9600Bd
		.db	00h 
;
;----------------------------------------------------------
; Multi-IO-Karte initialisieren
;		
mioini:	mvi	a,bap1	; PIO-Betriebsarten einstellen
		out	pio1s		;
		mvi	a,bap2	;
		out	pio2s		;
		xra	a		;
		out	pio2b		;
		out	pio2c		;
		;
		mvi	a,03h		; Reset com1 1
		out	com1s		;	
		mvi	a,bacom1	; com1-Betriebsart
		out	com1s		;
		mvi	a,55h		; RST On
		out	com1s		;
		;
		mvi	a,03h		; Reset com2 2
		out	com2s		;	
		mvi	a,bacom2	; com2-Betriebsart
		out	com2s		;
		mvi	a,55h		; RST On
		out	com2s		;	
		;
		mvi	a,bati0	; Timer-Betriebsarten
		out	tims		;
		mvi	a,bati1	;
		out	tims		;
		mvi	a,bati2	;
		out	tims		;
		;
		lxi	d,baud	; siehe oben
		mov	a,e		; in Timer 0 laden
		out	tim0		;
		mov	a,d		;
		out	tim0		;
		;
		mov	a,e		; in Timer 1 laden
		out	tim1		;
		mov	a,d		;
		out	tim1		;
		;
		mov	a,e		; in Timer 2 laden
		out	tim2		;
		mov	a,d		;
		out	tim2		;
		;
		ret	
;	
;====================================================================
; 
; Casettenausgabe
; 
fctcot:	lhld	adres		; letzten Parameter holen
		shld	srcend	; und ablegen
;
;
casot_m:	lda	srcbeg	; Abfrage wg. Abgleich
		cpi	0adh		; ad=Adjust
		jz	adjust	; ADJUST
		call	csinit	; Cas-Ausgabe	
		lxi 	b,0258h	; = 600d x 8,3ms
outlop:	xra	a
		call	casot1	; Casot1 Byte ausgeben
		dcx	b
		cmp	c
		jnz	outlop
		cmp	b
		jnz	outlop
		mvi	a,0d3h	; Startzeichen
		call	casot2	; CASOT2 3mal ausgeben
		call	casot1	; CASOT1
		xra	a
		call	casot1	; CASOT1 "00" als Abschlu
		lhld	srcend	; Parameter holen
		inx	h
		xchg
		lhld	srcbeg
output:	mov	c,m		; OUTPUT Zeichen holen
		call	po_m		; ausgeben
		inx	h
		mov	a,l		; Endabfrage
		cmp	e
		jnz	output	; weiter
		mov	a,h
		cmp	d
		jnz	output	; weiter
		xra	a
		call	casot2	; CASOT2
		call	casot2	; CASOT2
		call	lonit_m
		ret
;
; Casetteneingabe
;
; Programm mit 5s Vorspann laden
;
casin_m:	call csinit		;
		mvi	b,64h		; 100d Bytes erfassen
inlopo:	mvi	c,0ah		; 10d Bytes vorlauf
inlpo:	call	ri_m
		cpi	00h
		jz	inlo		; "00" erkannt
		dcr	b
		jnz	inlopo
		jmp	invert
		;
inlo:		dcr	c
		jnz	inlpo
		jmp	inlop2	; auf Startmaske "D3" warten
					;
invert:	mvi	a,0c3h
		out	cs_mode
		mvi	b,64h		; 100Bytes erfassen
inlop1:	mvi	c,0ah		; 10Bytes vorlauf
inlp1:	call	ri_m
		cpi	00h
		jz	inl1		; "00" erkannt
		dcr	b
		jnz	inlop1
		mvi	a,00h		; Vorspannfehler	
		jmp	inend		; Fehler: (Z)=1
inl1:		dcr	c
		jnz	inlp1		;
inlop2:	mvi	c,03h		; 3 StartBytes
inlop3:	call	ri_m
		cpi	0d3h
		jnz	inlop2	; "D3" nicht erkannt
		dcr	c
		jnz	inlop3	;
		jmp	inlop4	; 3x "D3" erkannt
		;
		mvi	a,01h		; Startzeichenfehler
		jmp	inend		; Fehler (Z)=1
		;
inlop4:	call	ri_m		;
		cpi	00h
		jz	input		; "00" erkannt
		xra	a
		mvi	a,02h		; Synchron Fehler
		jmp	inend		; Fehler (Z)=1
					;
input:	lhld	srcend
		inx	h
		xchg
		lhld	srcbeg
inlp:		call	ri_m		; Zeichen einlesen
		rz			; ENDE bei Datenverlust
		mov	m,a		; OK, Daten ins RAM ablegen
		inx	h
		mov	a,l		; Endabfrage
		cmp	e
		jnz	inlp		; nein, weiter
		mov	a,h
		cmp	d
		jnz	inlp		; nein, weiter
		xra	a
		inr	a		; OK (Z)=0
inend:	push	psw
		call	lonit_m
		pop	psw
		ret			;
;
;====================================================================
;
; Casetteninterface abgleich
;
; 		Testpunkt TP0 an Masse legen, TP1 mit
;		Timereingang G0 verbinden und Abgleichroutine aufrufen:
; 			ingabe "o ad", RETURN
; 		("o" = Casettenausgabe, "ad" fr Adjust)
;		Sollwerte:	4,000 Mhz 3750d
;				4,194 MHz	3690d
;				6,000 MHz	3125d
;				6,144 MHz	3080d
;				8,000 MHz	2500d
;
adjust:	mvi	a,31h		; Mod 0, BCD
		out	timer_ctl	; als BCD Zaehler
		mvi	a,00h
		out	timer_0
		mvi	a,50h		; Anfangswert 5000d 
		out	timer_0
		mvi	a,01h		; XOR triggern
		out	cs_mode		
		xra	a		; 
		out	cs_mode
		call	dely1	; 1ms warten
		mvi	a,01h		; Timer lesen
		out	timer_ctl	;
		in	timer_0	; low Byte
		mov	l,a
		in	timer_0	; high Byte
		mov	h,a
		xchg
		lhld	update	; Cursor Update
		push	h
		xchg
		call	adrot_v	; Zaehlerstand anzeigen
		pop	h
		shld	update
		call	onesec	; beruhigen
		jmp	adjust
;	
; CASOT1/2 gibt 1 bzw. 2Bytes ueber das Magnetbandinterface aus
;
casot2:	call	casot1
casot1:	push	b
		mov	c,a
		push	psw
		call	po_m		; Zeichen ausgeben
		pop	psw
		pop	b
		ret
;
;====================================================================
;
; RI_m 	liest ein Byte ueber das Magnetbandinterface ein (A)
;	
ri_m:		push	b		;
		mvi	c,0ch		; max 12ms warten
rilop: 	in	acia_ctl	; warten auf Daten
		ani	01h		; Empfangsregister voll
		jnz	ricte
		call	dely1
		dcr	c
		jnz	rilop		; maximal 12ms (C)
		mvi	a,03h		; Zeitueberschreitung 
		pop	b
		ret			; Fehler (Z)=1
ricte:	in	acia_dat	; Daten einlesen
		pop	b
		ret			; OK Zeichen in (A)
;
; PO_m 	gibt ein Zeichen ueber das Magnetbandinterface aus (C)
;	
po_m:		mov	b,c		; Daten (C) an kassetteninterface
		mvi	c,0ch		; Wartezeit 12ms
polop:	in	acia_ctl	; warten bis
		ani	02h		; Senderegister leer
		jnz	pocte
		call	dely1		; maximal 12ms (C)
		dcr	c
		jnz	polop
		mvi	a,04h		; Gegenstelle nicht bereit
		ret			; Fehler (Z)=1
pocte:	mov	a,b		; Daten ausgeben
		out	acia_dat
		ret			; OK
;
; Daten Ausgabe je nach gewaehlter Schnittstelle
; (2FC8h) Bit 7 6 5 4 bestimmen den kanal
;
;	Bit7	= V24
;	Bit6	= Centronis
;	Bit5	= Paralelle Ausgabe
;	Bit4	= Thermodrucker;
; 
lo_m:		lda	ioflg		; Daten (C) an Drucker mit 4800Bd
		ani	0f0h		; Bit 4-7 maskieren
		rlc
		jc	lo7		; LO7 = V24
		rlc
		jc	lo6		; LO6 = Centronics
		rlc
		jc	lo5		; LO5 = Paralele Schnittstelle
		rlc
		jc	lo4		; LO4 = Thermodrucker
		xra	a
		mvi	a,05h		; I/O Byte undefiniert
		ret
;
timout8	.equ	2000h		; ca 8000ms = 8s
;	
lo7:		push	b		; V24 schreiben
		lxi	b,timout8	;
lo7lop:	in acia_ctl
		ani	02h		; Statusbit maskieren
		jnz	lo7cts	; Senderegister leer	
		call	dely1		; Wartezeit max 2sek (BC)
		dcx	b
		xra	a
		cmp	b
		jnz	lo7lop
		cmp	c
		jnz	lo7lop
		pop	b
		mvi	a,07h		; Zeitueberschreitung 
		ret			; Gegenstelle nicht bereit
;
lo7cts:	in	acia_ctl
		ani	08h
		jnz	lo7cts	; warten CTS Eingang
lo_cte:	lxi	b,0000h	; Flags loeschen	
		push	b
		pop	psw
		pop	b
		mov	a,c
		out	acia_dat	; Daten ausgeben
		ret
;
;====================================================================
;
; Daten an Centronis Schnittstelle ausgeben
; Port CA,BA
;
lo6:		push	b
		lxi	b,timout
lo6lop:	in	0cah		; Centronics BUSY
		ani	40h
		jnz	lo6cte
		call	dely1
		dcx	b
		xra	a		; Endabfrage
		cmp	b
		jnz	lo6lop
		cmp	c
		jnz	lo6lop
		pop	b
		mvi	a,06h		; Gegenstelle nicht bereit
		ret			; FEhler (Z)=1
lo6cte:	pop	b
		mov	a,c
		out	0cah		; Daten an Centronics
		out	0bah		; Strobe ausgeben
		ret
;
; Ausgabe an Paralelle Schnittstelle 8255
;	
lo5:		push	b
		lxi	b,timout	;
lo5lop:	in	par_pc	; BUSY Abfrage
		ani	80h
		jz	lo5cte	;
		call	dely1
		dcx	b
		xra	a		; Endabfrage
		cmp	b
		jnz	lo5lop
		cmp	c
		jnz	lo5lop
		pop	b
		mvi	a,08h		; Gegenstelle nicht bereit
		ret			; Fehler (Z)=1
;
lo5cte:	pop	b
		mov	a,c
		out	par_pa	; Daten ausgeben
		xra	a
		out	par_pc	; Strobe 	
		call	lo5end	; Wartezeit
		inr	a
		out	par_pc
lo5end:	ret			;
;
; Ausgabe auf Thermodrucker
;
lo4:		mov	a,c		;
		cpi	08h		; BS?
		jz	thddel
		cpi	0ch		; FF?
		jz	thdff
		cpi	0dh		; CR?
		jz	thdcr
		cpi	7fh		; DEL?
		jz	thddel
		cpi	20h		; Code im zulaessigen Bereich ?
		jc	lo4ret
		cpi	0a0h		; LF?
		jc	lo4cte
lo4ret:	nop
		mvi	a,09h		; unzulaessiger Code
		ret			; Fehler (Z)=1
;	
lo4cte:	lhld	buff_poi	;
		mov	m,c		; Zeichen in Textbuffer
		inx	h		; Bufferzeiger erhoehen
		shld	buff_poi
		lda	buff_num	; Zeichenzaehler erhoehen
		inr	a
		sta	buff_num	;
		cpi	14h		; Buffer voll ?
		rnz			; OK
		;	
thdcr:	xra	a		; Zeichenzaehler loeschen
		sta	buff_num
		lxi	h,buff_txt	; Zeiger auf Textanfang
		shld	buff_poi
		call	thermo	; Sprung nach Adr 2014h 
					; Thermodrucker ?
thdff:	mvi	a,14h		; Buffer loeschen
		lxi	h,buff_txt	;
thdffl:	mvi	m,20h
		inx	h
		dcr	a
		jnz	thdffl	; fertig ?
		inr	a
		ret
		;
thddel:	lda	buff_num	; Zeichen loeschen
		dcr	a
		rm			; Buffer war leer
		sta	buff_num
		lhld	buff_poi
		dcx	h
		mvi	m,20h		; mit "Blank" ueberschreiben
		shld	buff_poi
		ret
;
; Serielle Schnittstelle der CPU ansteuern
; SID Geraet A mit SOD Geraet B
; SOD Geraet A mit SID Gereat B
; und GND verbinden
;
fctsod:	lhld	adres		; SOD Ausgabe
		inx	h
		xchg			; (DE) Endadress
		lhld	srcbeg	; (HL) Startadresse
sodlop:	mov	c,m		; Zeichen holen
		call	cpuot		; und ausgeben
		inx	h
		mov	a,l		; Endabfrage
		cmp	e
		jnz	sodlop	; weiter
		mov	a,h
		cmp	d
		jnz	sodlop	; weiter
		ret
;		
fctsid:	lhld	adres	; SID eingabe
		inx	h
		xchg			; (DE) Endadresse
		lhld	srcbeg	; (HL) Staradresse
		mvi	a,0c0h	; SOD auf High
		sim			;
sidlop:	call	cpuin		; Zeichen holen
		mov	m,a		; ablegen
		inx	h
		mov	a,l		; Endabfrage
		cmp	e
		jnz	sidlop	; weiter
		mov	a,h
		cmp	d
		jnz	sidlop	; Weiter
		ret
;
; CPUOT	Ausgabe (C) ueber die CPU-Schnittstelle SOD
;
cpuot:	rim			; Ready abfragen
		ana	a
		jp	cpuot		; SID auf LOW, warten
		mov	a,c		; Zeichen holen
		call	stabit	; Startbit ausgeben
		mvi	c,08h		; Bitzaehler
otlop:	rrc			; LSB zuerst ausgeben
		ana	a
		jm	high		; Bit ist High
low:		call	stabit	; LOW-Bit ausgeben	
		jmp	outcte	;
			
high:		call	stobit	; High-Bit ausgeben
outcte:	dcr	c
		jnz	otlop		;
		call	stobit	; Stopbit ausgeben
		ret			;
stabit:	push	psw
		mvi	a,0c0h	; SOD auf High
		jmp	bitcte	;
stobit:	push	psw
		mvi	a,40h		; SOD auf Low
bitcte:	sim
		call	dely2		; Wartezeit
		pop	psw
		ret
;
; CPUIN 	Daten ueber die CPU-Schnittstelle entgegennehmen SID
;
cpuin:	rim			; SID abfragen
		ana	a
		jp	cpuin		; SID auf low, warten
		push	d
		mvi	d,08h		; Bitzaehler
		mvi	e,00h		; Schieberegister
		call	dely1		; halbe Bit-Dauer warten
inlop:	call	dely2		; mitte Datenbit
		rim
		ani	80h		; MSB maskieren
		ora	e		; verknpfen
		rrc
		mov	e,a		; ins Schieberegister
		dcr	d
		jnz	inlop		;
		rlc
		push	psw
		call	dely2		; mitte Stopbit
		pop	psw
		pop	d
		ret
		;
dely2:	call	dely1		; 2ms warten
		call	dely1
		ret
;
;====================================================================
;
; Geraetetreiber fuer schnelle V24 auf CF-Karte
;
; Befehle fuer AVR
;
bcfres	.equ	00h		; AVR-Reset
bcfst		.equ	0f0h		; AVR-Status
bcfstart	.equ	50h		; RTS loeschen
bcfstop	.equ	51h		; RTS setzen
bcftx		.equ	52h		; ein byte senden		ohne Buffe
bcfrx		.equ	53h		; ein byte empfangen
bcfrxs	.equ	55h		; Zeile empfangen bis CR (max 64Zeichen)
bcftxz	.equ	56h		; ein Byte senden (mit Timeout 10s)
bcfrxz	.equ	57h		; ein byte empfangen mit Timeout 10s
bcfvst 	.equ	5Fh		; Status der V24 Schnittstelle
					; Bit 3 CTS Signal
					; Bit 4 RTS Signal
					; Bit 5 Senderegister leer
					; Bit 7 Empfangsregister voll
;
;
; TX/RX einschalten
;
cv24start:	mvi	a,bcfstart	;
		call	piowr		;
		ret			;
;
; TX/RX ausschalten
;
cv24stop:	mvi	a,bcfstop	;
		call	piowr		;
		ret			;
;
; Byte ueber V24 Senden
; (C) an CF-Karte
cv24tx:	mvi 	a,bcftx	; Befehl
		call	piowr		;
		mov 	a,c		; Zeichen	
		call	piowr		; ausgeben
		ret			;
;
; Byte von V24 holen
; (A) von CF-Karte
cv24rx:	mvi 	a,bcfrx	; Befehl
		call	piowr		;
		call	piord		; Zeichen holen
		ret
;
;
; Byte ueber V24 Senden (mit Timeout)
; (C) an CF-Karte
cv24txz:	mvi 	a,bcftxz	; Befehl
		call	piowr		;
		mov 	a,c		; Zeichen	
		call	piowr		; ausgeben
		ret			;
;
; Byte von V24 holen mit Timeout nach 10s
; (A) von CF-Karte
cv24rxz:	mvi 	a,bcfrxz	; Befehl
		call	piowr		;
		call	piord		; Zeichen holen
		ret
;
; V24 Statusabfrage 	ost= OutputStatus
;				ist= InputStatus
;	
; (A)=00h Schnittstelle belegt
; (A)=FFh Schnittstelle frei	
cv24ost:	mvi	a,bcfvst	; Status fr Ausgabe
		call	piowr		; abfragen
		call	piord		;
		ani 	00100000b	; Bit 5 maskieren
		jz  	cv24oe	; kein Zeichen vorhanden
		mvi 	a,0ffh	; Zeichen im Eingangsbuffer
cv24oe:	ret
;
; (A)=00h kein Zeichen 
; (A)=FFh Zeichen wartet	
cv24ist:	mvi 	a,bcfvst	; Status fr Eingabe
		call	piowr		; abfragen
		call	piord		;
		ani	10000000b	; Bit 7 Maskieren
		jz	cv24ie	; kein zeichen
		mvi	a,0ffh	; Zeichen im Eingangsbuffer
cv24ie:  	ret			;	
;
; Zeile empfangen (bis CR max 64 Zeichen)
;
cv24rxs:	lxi	h,zbuff	; Zeilenbuffer
		mvi	b,40h		; maxbuffer
		mvi	a,bcfrxs	; 
		call	piowr		;
	
str_l:	call	piord		; Byte holen
		mov 	m,a		;      speichern
		cpi 	0dh		; ? CR
		jz  	str_end	; Zeilenende
		inx 	h
		dcr 	b		;
		jnz 	str_l		; weiter
str_er:	mvi 	a,01h		; Fehler Bufferueberlauf
		ret			; Fehlercode 01h

str_end:	xra a			; Zeile vollstndig empfangen
		ret			; Fehlercode 00h
;		
;
;
; PIO-A lesen (A)
; wartet bis Daten bereitstehen 

piord:	in	cf_c		; Status ermitteln
		ani	00100000b	; IBF maskieren
		jz	piord		; warte auf Daten
		in	cf_a		; daten lesen
		ret			; (A) Daten
;
; (A) PIO-A schreiben
; 
piowr:	push	psw		;
pio_sts:	in	cf_c		; Status ermitteln
		ani	10000000b	; /OBF maskieren
		jz	pio_sts	; warten auf Quittung /ACK
		pop	psw
		out	cf_a		; Daten schreiben
		ret			; 
;
;====================================================
;
; cfinit = Reset Befehl an CF-Karte
;          
cfinit:	call	cfpini	; PIO und AVR initialisieren
		mvi	a,bcfres	; Test ob CF-Erweiterung vorhanden ist;
		call	piowr		;
		xra	a		;
		call	piord		;
		cpi	00h		; 	
		jnz	cferr		; CF nicht bereit -> Fehler
		ret			; CF-Interface OK
;
; PIO Initialisieren mit Betriebsart
; Kanal A bidirektional
; Kanal C Handshake
; Kanal B Ausgabe
;
cfpini:	push	psw		; PIO Betriebsart einstellen
		mvi	a,cf_ba	; equ	0c1h
		out	cf_ctl	; 
		pop	psw		;
		ret			;
;
; avr_stat = Status CF-Karte ermitteln 0=OK
;            sonst Fehlermeldung
;
cfstat:	mvi	a,bcfst	; Status AVR ermitteln
		call	piowr		; 
		call	piord		;
		ret			;
;
cferr:	mvi	a,01h		; Fehler CF-Karte
		ora	a		;
		ret			;
;
;
;====================================================================
;
; Consolenausgabe mit Filter fuer Sonderzeichen
;
; Anpassung des TEXASS-Textverarbeitungsprogramm
;
; Erweiterung des Mode-Flags
;
;	Bit7 6 5 4 3 2 1 0
;	   I I I I I I I I
;        I I I I I I I + - Daten-Mode
;        I I I I I I + - - Adress-Mode
;        I I I I I + - - - Register-Mode
;        I I I I + - - - - Funktionseingabe
;	   I I I + - - - - - ESC-0 wird von "1B" ESC gesetzt
;	   I I + - - - - - - ESC-1 zweiter Parameter
;	   I I + - - - - - - ESC-2 dritter Parameter
;	   + - - - - - - - - Einzelschrittbetrieb
;
co_m:		push	psw		; ESCAPE
		push	h
		push	d
		push	b
		mvi	a,0c3h	; Hilfszellen fuer Sprungziel
		sta	brkpt1	; JMP 1006h Videomonitor
		lxi	h,co_v	; bzw. je nach Sondertasten
		shld	hbufe		;
		lda	modflg
		mov	b,a
		ani	10h		; ESC-Flag abfrageb
		jnz	estest	; gesetzt, weiter
		mov	a,c		; ASCII
		cpi	1bh		; = ESC ?
		jz	escset	; ja, ESC-Flag setzen
		mov	a,b		;
		jmp	escend	;
					;	
estest:	mov	a,b		;
		ani	40h		; ESC-Flag 2 abfragen
		jz	escte1	; gesetzt, weiter
		mov	a,c		;
		sta	hbuf3		; CO-Byte ablegen
		mvi	c,16h		; = TSCPOS
		jmp	esprcs	;
;
escte1:	mov	a,b		;
		ani	20h		; ESC-Flag 1 abfragen
		jz	escte2	; geloescht, weiter
		mov	a,c
		sta	hbuf3		; CO-byte ablegen
		mov	a,b
		ori	40h		; ESC-Flag 2 setzen
		jmp	escend
;	
escte2:	mov	a,c		; ASCII
		cpi	16h		; TSCPOS?
		jnz	esprcs	; nein, weiter
		mov	a,b
		ori	20h		; ESC-Flag 1 setzen
		jmp	escend
;
esprcs:	mov	a,c		; ASCII
		cpi	20h		; kleinstes ASCII zeichen	
		jnc	escl		; ist ASCII, weiter
		cpi	1ah		; oberer Grenzwert
		jnc	esclr		; ueberschritten, weiter
		cpi	10h		; unterer Grenzwert
		jc	esclr		; unterschritten, weiter
		lxi	h,esclr	; CTE-Adresse
		push	h		; sichern
		lxi	h,cojt1	; 1.Adresse aus JMP Tabelle
		sui	10h		; ASCII-Wert - 10
		rlc			;  * 2
		mov	e,a		; 
		mvi	d,00h		; (DE) Versatz
		dad	d		; auf JMP-Tabelle
		mov	e,m		; neue Adresse aus JMP-Tabelle laden
		inx	h		;
		mov	d,m		;
		xchg			; (HL) Zieladresse
		pchl			; Sprung zu Zieladresse 
;	
escl:		lda	modflg
		ani	8fh
		jmp	escend
;	
esclr:	lda	modflg
		ani	8fh		; ESC-Flags loeschen
escnd:	lxi	h,escret	;  = RET
		shld	hbufe		; in Hilfszelle
escend:	sta modflg
		pop	b
		pop	d
		pop	h
		pop	psw
		jmp	brkpt1	; Sprung ueber Hilfszellen
					; 
;	
escset:	mov	a,b
		ori	10h
		jmp	escnd		;
;
; Sprungtabelle je nach ASCII-Code
;
; Labels als CO JMP TAB n vergeben
;             "  "   "  " z = Ziel 
;
	
cojt1:	.dw	clrvid_v	; ESC-P = Spung nach Clearvid 102Dh
;        	
cojt2:	.dw	tchome	; ESC-Q = Cursor Home
;	
cojt3:	.dw	crsoff_v  	; ESC-R = Cursor aus
;	
cojt4:	.dw	crson_v	; ESC-S = Cursor Ein
;		
cojt5:	.dw	cojt5z	; ESC-T = Scroll up
;		
cojt6:	.dw	cojt6z	; ESC-U = Scroll down
		
cojt7:	.dw	cojt7z	; ESC-V = Cursor Position (LL/CC)
		
cojt8:	.dw	cojt8z	; ESC-W = Schirm ab Cursor loeschen
;		
cojt9:	.dw	cojt9z	; ESC-X = Zeile loeschen
;	
cojt10:	.dw	cojt10z	; ESC-Y = Zeile ab Cursor loeschen
;	
tchome:	lxi	h,vhome	;
		jmp	crspos_v	;
;	
cojt5z:	lxi	b,bcup	;
		lxi	d,deup
		lxi	h,hlup
uplop:	in	vid_dspen	; Ausgabe mit Strahlrueklauf
		rrc			; Synchronisieren
		jnc	uplop		; Blanking abwarten
		mov	a,m
		stax	b
		inx	b
		inx	h
		mov	a,e		; Endabfrage
		cmp	l
		jnz	uplop
		mov	a,d
		cmp	h
		jnz	uplop
		ret
;	
cojt6z:	lxi	b,bcdwn	;
		lxi	d,dedwn
		lxi	h,hldwn
dwnlop:	in	vid_dspen 	; Ausgabe mit Strahlrueklauf
		rrc			; Synchronisieren
		jnc	dwnlop	; Blanking abwarten
		mov	a,m
		stax	b
		dcx	b
		dcx	h
		mov	a,e		; Endabfrage
		cmp	l
		jnz	dwnlop
		mov	a,d
		cmp	h
		jnz	dwnlop
escret:	ret
;	
cojt7z:	lhld	hbuf3		
		xchg
		lxi	h,vidram	; VIDO-RAM Start - eine zeile
		lxi	b,lin		; Zeilenlaenge (80 Zeichen)
poslop:	dad	b
		dcr	e
		jnz	poslop	; Schleife bis Zeile erricht ist
		mov	c,d
		dad	b		; Zeilenposition 
		call	crspos_v
		ret
;	
cojt8z:	lxi	d,vidend	; Bildschirm ab Cursor loeschen
		lhld	update
		jmp	filop
;	
cojt9z:	call	clrsub	; Zeile loeschen
		jmp	clrlin_v
;	
cojt10z:	call	clrsub	; Zeile ab Cursor loeschen
		lxi	b,lin
		dad	b
		xchg
		lhld	update
filop:	in	vid_dspen	; Ausgabe mit Strahlrueklauf 
		rrc			; Synchronisieren
		jnc	filop		; auf Blanking warten
		mvi	m,20h		; ASCII Leerzeichen "20"
		inx	h
		mov	a,e
		cmp	l		; Endabfrage
		jnz	filop
		mov	a,d
		cmp	h
		jnz	filop
		ret
;	
clrsub:	lxi	b,lininv	; (BC) negative Zeilenlaenge
		lxi	d,vidend
		lhld	update
		xchg
clrlop:	mov	a,h		; (HL) Zeilenanfang
		cmp	d
		jc	clrend
		jnz	clrcte
		mov	a,l
		cmp	e
		jc	clrend
		jz	clrend
clrcte:	dad	b
		jmp	clrlop
;	
clrend:	ret			; (HL) Anfang Cursorzeile
;
;====================================================================
;
; Echzeituhr mit RTC-58321
;
; Betriebsarten PIO-2:
; bap2		= 82h	 1000 0010 (nach Reset)
;			 Kanal A Modus 0 Ausgabe 
;			 Kanal B Modus 0 Eingabe 
;			 Kanal C Ausgabe
; bap2rdu	= 92h	 1001 0010 (Uhr lesen)
;			 Kanal A Modus 0 Eingabe 
;			 Kanal B Modus 0 Eingabe 
;			 Kanal C Ausgabe
;
; Daten/Adressen ueber Port pio2a = 43h
; 	Bit 7 6 5 4 3 2 1 0
;	    x x x x + + + + Daten 0-3
; 
;       Bit 4-7 frei
;	      
; Steuerung ueber Port pio2c = 63h
;
; 	Bit 7 6 5 4 3 2 1 0
;	    x x x I I I	I I
;	    x x x I I I I + WR
;	    x x x I I I + - RD
;	    x x x I I + - - ADR-WR
;	    x x x I + - - - CS
;	    x x x + - - - - Stop
;
; RTC Registeradressen
; Adressen	Bedeutung		Gueltigkeit Datenbit 0-3
;
; 	3 2 1 0
; 0	0 0 0 0	= Sekunden-Einer	0 - 9
; 1	0 0 0 1	= Sekunden-Zehner	0 - 5
; 2	0 0 1 0	= Minuten-Einer	0 - 9
; 3	0 0 1 1	= Minuten-Zehner	0 - 5
; 4	0 1 0 0	= Stunden-Einer	0 - 9
; 5	0 1 0 1	= Stunden-Zehner	0 - 1 / 0 - 2 im 24Std-Modus
; 6	0 1 1 0	= Wochentag		0 - 6
; 7	0 1 1 1	= Tag-Einer		0 - 9
; 8	1 0 0 0	= Tag-Zehner	0 - 3 (D2 = 1 fuer Februar mit 29Tagen/sonst 28Tage)
; 9	1 0 0 1	= Monat-Einer	0 - 9
; A	1 0 1 0	= Monat-Zehner	0 - 1
; B	1 0 1 1	= Jahr-Einer	0 - 9
; C	1 1 0 0	= Jahr-Zehner	0 - 9
; 	
;====================================================================
; Modul Anfang
; mio-uhr-18-1.asm
;
; Modul Echtzeit-Uhr auf Multi-IO-Karte
;
; Stand:
;		07.02.2018		; Modul zusammengestellt
;
;
;--------------------------------------------------------------------
;
; Portzuweisung (neus Label fuer die Systemuhrabfrage)
;
rtc_dat	.equ	pio2a		; 4bit Datenregister
rtc_ctrl	.equ	pio2c		; RTC-Steuerregister
;					;
;					; Bit
;					; 7654 3210
;					;
rtccs		.equ	18h		; 0001 1000	= RTC Selektieren und Zaehler-Stop
rtcwr		.equ	19h		; 0001 1001	= RTC Datennibble schreiben
rtcrd		.equ	1ah		; 0001 1010	= RTC Datennibble lesen
rtcawr	.equ	1ch		; 0001 1100	= RTC Registeradresse schreiben
;
zubuff	.equ	buff_poi	; 2e00h
;
;--------------------------------------------------------------------
;
wr_uhr:	call	wr_uhrbuff	; Parameter uebergeben
		call	uhr_set	; RTC mit Daten fuellen
		ret			;

rd_uhr:	call	rd_rtc	; RTC auslesen Daten in (uhrbuff)
		ret			;
		
time_m:	call	rd_rtc	; Uhrzeit als String
		call	uhr_asci	; in (uhrasc) ablegen
		lxi	h,uhrasc	; Uebergabe Startadresse String
		ret			;
					;
dely10:	ret			; ca. 10s verzoegerung		
;--------------------------------------------------------------------
;
; Parameter aus SRCBEG,-End u. DSTBEG in 
; Uhrenbuffer (uhrbuff) als BCD ablegen
;
wr_uhrbuff:	lxi	h,uhrbuff	; 
		xra	a		; Sekunden loeschen
		mov	m,a		;
		inx	h		;
		mov	m,a		;
		inx	h		;
		shld	zubuff	; Uhrenbuffer
		
		lhld	srcbeg	; Std/Min
		mov	a,l		;
		call	uhr_ni	;
		mov	a,h		;
		call	uhr_ni	;
		
		lhld	srcend	; wTag/Tag
		mov	a,h		;
		ani	07h		;
		call	wr_ubuff	;
		mov	a,l		;
		call	uhr_ni	;

		lhld	dstbeg	; Mon/Jahr
		mov	a,h		;
		call	uhr_ni	;
		mov	a,l		;
		call	uhr_ni	;
	
		ret			;
;
;
; Daten in Nibble aufspalten und an Buffer senden
;	
uhr_ni:	push	psw
		ani	0fh
		call	wr_ubuff	; unteres Nibble an Buffer
		pop	psw
		rrc
		rrc
		rrc
		rrc
		ani	0fh
		call	wr_ubuff	; oberes Nibble an Buffer
		ret
;
wr_ubuff	push	h
		lhld	zubuff	; Daten in Uhrbuffer ablegen	
		mov	m,a		;
		inx	h		;
		shld	zubuff	;
		pop	h
		ret
;
;--------------------------------------------------------------------
;
; Uhrzeit stellen
; RTC-58321 mit BCD-Daten aus dem Uhrbuffer (uhrbuff) laden
;
uhr_set:	lxi	h,uhrbuff	; 
		mvi	c,00h		; Zeiger fuer RTC-Register
		mvi	b,0dh		; Zaheler
		
		mvi	a,bap2	; Ausgabe
		out	pio2s		;
		
uhr_sl:	mov	a,c		;
		cpi	05h		;
		jz	uhrwr_24	;
		mov	a,m		; bit holen
uhr_sl2:	call	wr_rtc	;
		inx	h		; naechstes byte
		dcr	b		;
		jnz	uhr_sl	;
		xra	a		;
		out	rtc_ctrl	; RTC Steuereingaenge auf 0
		
		ret
		
uhrwr_24	mov	a,m		;
		ori	08h		; 24Std. Modus
		jmp	uhr_sl2	;
;
;	
; wr_rtc Daten in RTC-58321 schreiben
;
wr_rtc:	push	psw		; daten retten
		mov	a,c		; Registeradresse
		out	rtc_dat	;
		call	dely1_m	;
		mvi	a,rtcawr	; Adress-Schreibimpuls
		out	rtc_ctrl	;
		call	dely1_m	;
		mvi	a,rtccs	;
		out	rtc_ctrl	;
		
		call	dely1_m	;
		pop	psw		;
		out	rtc_dat	; Wert in RTC laden
		mvi	a,rtcwr	; Schreibimpuls
		out	rtc_ctrl	;
		call	dely1_m	;
		mvi	a,rtccs	;
		out	rtc_ctrl	;	
		
		inr	c		; naechste Adresse einstellen
		ret

;
;
;--------------------------------------------------------------------
;
; Uhrzeit aus RTC-58321 lesen
; BCD nach Uhr-Buffer (uhrbuff)
;
; (c) Registeradresse	
; (hl) Bufferadresse
;
rd_rtc:	lxi	h,uhrbuff	;
		mvi	c,00h		; Zeiger auf sek-einer
		mvi	a,bap2	; Ausgabe
		out	pio2s		;
		mvi	a,rtccs	; RTC stop/cs
		out	rtc_ctrl	;
urdloop:	call	dely1_m	;
		mov	a,c		;Adresse einstellen
		out	rtc_dat	;
			
		call	dely1_m	;
		mvi	a,rtcawr	; Adresseschreiben
		out	rtc_ctrl	; ADR-Schreibimpuls
		call	dely1_m	;
		mvi	a,rtccs	;
		out	rtc_ctrl	;
			
		mvi	a,bap2rdu	; Kanal auf Eingabe umschalten
		out	pio2s	
			
		call	dely1_m	; Register lesen
		mvi	a,rtcrd	;
		out	rtc_ctrl	;
		call	dely1_m	;
		in	rtc_dat	;
		ani	0fh		;oberes nibble ausblenden
		mov	m,a		;
		mov	a,c		;
		cpi	05h		; Std. Zehner
		jz	uhrrd_24	; 
urdl_2:	mvi	a,rtccs	;
		out	rtc_ctrl	; Leseimpuls beenden
			
		mvi	a,bap2	; Ausgabe
		out	pio2s		;
			
		inx	h		; naechste Adresse einstellen
		inr	c		;
		mov	a,c		; alle Register gelesen
		cpi	0dh		;
		jnz	urdloop	; nein
		
uhr_end:	mvi	a,00h		; ende
		out	rtc_ctrl	; 
		ret			;
;
uhrrd_24:	mov	a,m		; 24Std. Flag ausblenden
		ani	03h		;
		mov	m,a		;
		jmp	urdl_2	;
;
;
;--------------------------------------------------------------------		
;
; Uhrzeit in ASCII String wandeln
; liest den (uhrbuff) aus und wandelt von BCD nach ASCII
; und legt die Uhrzeit als String in (uhrasc) ab
;
uhr_asci:	lxi	h,uhrasc	; Startadr. ASCII-Buffer
		lxi	d,15h		; Versatz
		dad	d		; HL auf ende ASCII-Buffer
		lxi	d,uhrbuff	;
		mvi	m,00h		; 00h eintragen fuer String-Ende
		dcx	h
		mvi	m,68h		; ASCII "h" speichern
		call	h2bcd		; Sekunden in ASCII Buffer 
		dcx	h
		mvi	m,3ah		; ASCII ":" (Trennzeichen)
		call	h2bcd		; Minuten in ASCII Buffer
		dcx	h
		mvi	m,3ah		; ASCII ":" 
		call	h2bcd		; Stunden in ASCII Buffer
		dcx	h
		mvi	m,3bh		; ASCII ";"
		lxi	d,uhrjare
		call	h2bcd		; Jahr in ASCII Buffer
		dcx	h
		mvi	m,2eh		; ASCII "."
		lxi	d,uhrmone
		call	h2bcd		; Monat in ASCII Buffer
		dcx	h
		mvi	m,2eh		; ASCII "."
		lxi	d,uhrwte
		call h2bcd		; Tag in ASCII Buffer
		dcx	h
		mvi	m,2ch		; ASCII ","
		dcx	h
		push	h		;
		lxi	h,wt_tab	; Tabelle fuer Wochentag 
		lda	uhrklte	; Wochentag holen
		ani	07h		; mit Tabelle umsetzen
		rlc			;
		mov	e,a		;
		mvi	d,00h		;
		dad	d		;
		mov	e,m		;
		inx	h		;
		mov	d,m		;
		pop	h		;
		mov	m,e		;
		dcx	h		;
		mov	m,d		;
		ret
;
;
; h2bcd = hole 2 BCD-Werte aus Buffer
;	
h2bcd:	call	hbcd		; 
;
hbcd:		dcx	h		;
		ldax	d
		adi	30h		; Gueltigkeit pruefen
		cpi	3ah
		jnc	hbcd_er	; Fehler
		cpi	30h
		jnc	hbcde		; OK Wert zueruckgeben
hbcd_er:	mvi	a,2dh		; Wert ungueltig "-" zurueckgeben
;	
hbcde:	mov	m,a
		inx	d		; naechste Bufferadresse
		ret
;
; Wochentage ..oMiDiMoDrFaSoS
; 1 = Montag, 7 = Sonntag 
; Tage sind in umgekehrter Reihenfolge abgelegt
; weil der ASCII-Buffer von hinten nach vorn 
; aufgebaut wird
;
wt_tab:	.db	52h		; "R"                
		.db	45h		; "E" - Error
		.db	6Fh		; "o" - 
		.db	4Dh		; "M" - Montag
		.db	69h		; "i"
		.db	44h		; "D"
		.db	69h		; "i"
		.db	4Dh		; "M"
		.db	6Fh		; "o"
		.db	44h		; "D"
		.db	72h		; "r"
		.db	46h		; "F"
		.db	61h		; "a"
		.db	53h		; "S"
		.db	6Fh		; "o"
		.db	53h		; "S"
;
;					
; Ende Modul mio-uhr-18-1.asm 
;====================================================================
;
; Monitorerweiterung
;
; Boot-Lader
; CF-Hilfsprogramme
; -lschen
; -Systemspuren von Diskette holen
; -Systemspuren auf CF-Karte schreiben
;
;-----------------------------------------------------------------
;
		.org 0b00h      	; Startadresse
         
cpmb		jmp	boot		; CP/M von CF-Karte starten
format:	jmp	form		; Systemspuren mit 00h beschreiben
sysrd:	jmp	sread		; Systemspuren 0-3 von Diskette in Buffer laden
syswr:	jmp	swrite	; Systemspuren auf CF-Karte schreiben
ende:		lxi	h,msge	; Endemeldung
		call	string_v	;
		jmp	warm_v	; zurueck zum Monitor
;
;
;-----------------------------------------------------------------
;
boot:		call	cfinit	; von CF-Karte
		jnz	booterr	; Fehler
		lxi	h,msg3	;
		call	string_v	; Meldung
		lxi	h,ccp60k	; Startadresse 60k CP/M
		shld	datadr	; - sichern
		mvi	a,0h		; Startspur = 0
		sta	trknr		; - sichern
		mvi	a,2h		; Startsektor = 2
		sta	secnr		; - sichern
		;
		mvi	a,81h		; Bank 1 ab 4000h
		out	bank		; einschalten
		;
bootl:	call	settr_f	; Spur setzen
		call	setsek_f	; Sektor setzen
		call	rdsek_f	; Sektor schreiben
		lda	secnr		;
		inr	a		; Sektornummer hochzaehlen
		inr	a		; wg. Sektorberechnung im AVR
		sta	secnr		;
		cpi	10h		; Sektor 16 erreicht
		jc	bootl		; nein - weiter
		mvi	a,1h		; Sektor 1
		sta	secnr		; - sichern
		lda	trknr		; Spurnummer holen
		inr	a		;
		sta	trknr		; - sichern
		cpi	03		;
		jc	bootl		; nein weiter
		;
		jmp	bios60k	; jmp bios60k 
		;
booterr:	lxi	h,errmsg1	; Fehler CF-Karte
		call	string_v
		jmp	warm_v
	
form:		call	cfinit	; init
		
		lxi	h,msg4	;
		call	string_v	; Meldung
		lxi	h,buffer	; Startadresse Buffer
		shld	datadr	; - sichern
		lxi	b,200h	; Zaehler
fillb:	mvi	a,00h		; Buffer mit 00h fuellen		
		mov	m,a		;
		inx	h		;
		dcr	c		;
		jnz	fillb		;
		dcr	b		;
		jnz	fillb		;
		;		
		mvi	a,0h		; Startspur = 0
		sta	trknr		; - sichern
		mvi	a,1h		; Startsektor = 1
		sta	secnr		; - sichern
		;
		;
forml:	lxi	h,buffer	; Startadresse Buffer
		shld	datadr	; - sichern
		call	settr_f		; Spur setzen
		call	setsek_f	; Sektor setzen
		call	wrsek_f	; Sektor schreiben
		lda	secnr		;
		inr	a		; Sektornummer hochzaehlen
		inr	a		; wg. Sektorberechnung im AVR
		sta	secnr		;
		cpi	10h		; Sektor 16 erreicht
		jc	forml		; nein - weiter
		mvi	a,1h		; Sektor 1
		sta	secnr		; - sichern
		lda	trknr		; Spurnummer holen
		inr	a		;
		sta	trknr		; - sichern
		cpi	4		; Spur 4 erreicht
		jc	forml		; nein - weiter
		jmp	ende		;

;-----------------------------------------------------------------
;
; Systemspuren 0-4 von Diskette LW 1 in Buffer laden
; ! nur 60k Version
;		
sread:	call	cfinit	; init
		;
		lxi	h,msg1	;
		call	string_v	; Meldung ausgeben
		mvi	b,1Ah		; Selektbyte fuer LW 1
		mvi	c,40		; Anzahl der Sektoren
		mvi	d,0h		; Startspur = 0
		mvi	e,2h		; Startsektor = 1
		lxi	h,buffer	; Startadresse Buffer
		call	rdblk_f		; Block einlesen
		jmp	ende		;
		;
;-----------------------------------------------------------------
;
; Buffer als Systemspuren auf CF schreiben
;	
swrite:	lxi	h,msg2	;
		call	string_v	; Meldung
		lxi	h,buffer	; Startadresse Buffer
		shld	datadr	; fuer 1.Sektor
		mvi	a,0h		; Startspur = 0
		sta	trknr		; - sichern
		mvi	a,2h		; Startsektor = 2
		sta	secnr		; - sichern
		;
copyl:	call	settr_f	; Spur setzen
		call	setsek_f	; Sektor setzen
		call	wrsek_f	; Sektor schreiben
		lda	secnr		;
		inr	a		; Sektornummer hochzaehlen
		inr	a		; wg. Sektorberechnung im AVR
		sta	secnr		;
		cpi	10h		; Sektor 16 erreicht
		jc	copyl		; nein - weiter
		mvi	a,1h		; Sektor 1
		sta	secnr		; - sichern
		lda	trknr		; Spurnummer holen
		inr	a		;
		sta	trknr		; - sichern
		cpi	03h		;
		jc	copyl		; nein weiter
		
		jmp	ende		;
		
;
;------------------------------------------------------------------------------
; Texte zu CF-Karte
;
msg0: 	.db	0Dh		; CR
		.db	0Ah		; LF               
		.db	"SYSCOPY"
		.db	" V 4.2"
		.db	" (C)W.Roemer"
		.db	0Dh		; 
		.db	00h		; Ende MSG0
;			
msg1: 	.db	0Dh		; 
		.db	0Ah		;                
		.db	"Systemspuren"
		.db	" von LW 1"
		.db	" in Buffer laden"
		.db	0Dh		; 
		.db	00h		; 
;	
msg2: 	.db	0Dh		; 
		.db	0Ah		;                
		.db	"Systemspuren"
		.db	" von Buffer"
		.db	" auf CF-Karte"
		.db	" schreiben"
		.db	0Dh		; 
		.db	00h		; 
;	
msg3: 	.db	0Dh		; 
		.db	0Ah		;                
		.db	"CP/M 2.2"
		.db	" von CF-Karte"
		.db	" starten"
		.db	0Dh		; 
		.db	00h		; 
		
msg4: 	.db	0Dh 		; 
		.db	0Ah		;                
		.db	" CF-Karte"
		.db	" loeschen"
		.db	0Dh		; 
		.db	00h		; 	
;		
msge: 	.db	0Dh
		.db	" fertig"
		.db	0Dh		; 
		.db	00h
;
;====================================================================
;
;Intel-Hex Lader
;
i_hex:	jmp	ihexl		; Intel-Hex empfangen, interpretieren

hexend:	jmp	warm_v	; zurueck zum Monitor
;
;--------------------------------------------------------------------
;
; Hauptprogramm
;
; - laden Intel-Hex-File Zeilenweise
; - Intel-Hex Interpreter
;

ihexl:	lxi	h,meld_0	; Meldung "Ladeprogramm" 
		call	string_v
		lxi	h,meld_1	; Start-Meldung Datenuebertragung 
		call	string_v
		call	cv24stop
		call	onesec	; Wartezeit bis Datenabgeholt werden
		
lade_l:	lxi	h,zbuff	; ZeilenBuffer max 64Byte
		call	cv24rxs	; Zeile empfangen
		cpi 	00h		; Fehler auswerten
		jnz 	lade_e4	; Bufferueberlauf
		;	
		lxi	h,zbuff	; ZeilenBuffer max 64Byte
		;	
		call	hxi_		; eine Zeile interpretieren und im  Zielbuffer ablegen
		shld	dstend	; Test
		;
		mvi	c,2ah		; Sternchen pro Zeile ausgeben
		call	co		;
		;
		cpi	01h		; Fehlercode auswerten
		jz	lade_e1	; kein ASCII
		cpi	02h		;
		jz	lade_e2	; keine Startmarke
		cpi	03h		;
		jz	lade_e3	; falsche Pruefsumme
		cpi	04h		;
		jnz	lade_l	; weiter bis Dateiende
		;
		lxi	h, meld_5	; alles OK zurueck zum Monitor
lade_e:	call	string_v	; Bildschirmmeldung ausgeben
		call	cv24start	;
		call	onesec
		jmp	hexend
		
lade_e0:	lxi	h,err_1	;
		jmp	lade_e	;
		
lade_e1:	lxi	h,err_1	;
		jmp	lade_e	;
		
lade_e2:	lxi	h,err_2	;
		jmp	lade_e	;
		
lade_e3:	lxi	h,err_3	;
		jmp	lade_e
	
lade_e4:	lxi	h,err_0	;
		jmp	lade_e
	
;
;
;==============================================================================
; Eine Zeile Intel-Hex Daten aus RS232 Buffer lesen
;
; interpretieren und
; in Buffer legen
;
; eine Intel-Hex Zeile in 8Bit Systemen 
; ! jedes Byte ist in zwei ASCII-Zeichen codiert (z.B.10 = 31 30) !
;
;   I   Daten fuer Pruefsumme addieren inkl Pruefsumme = 0         I
;   I                                                           I
; : 10 2010 00 66 20 C3 88 23 C3 3B 24 D3 4B C9 1A D3 4B 13 C9 AF 0D 0A
; I I  I    I  I                                               I  I  I
; I I  I    I  1.Byte                                          I  I  Line-Feed
; I I  I    I                                                  I  I
; I I  I    Typ 00=Nutzdaten                                   I    CR
; I I  I        01=Ende der Inte-Hex datei (EOF)               I
; I I  I                                                       Pruefsumme
; I I  Startadresse
; I I
; I Anzahl Bytes
; I
; Startmarke
;
;
; Registerzuweisung:
;	
; (HL) Zeiger auf Lesebuffer
; (DE) Zeiger auf Schreibbuffer
; (C)  Bytezaehler
; (B)  Pruefsumme
;
;
;----------------------------------------------------
; Hex Interpreter wandelt eine Zeile Intel-HEX in Binaer-Code
; Anmerkung: Adressen werden pro Zeile ausgewertet, Zwischenraueme
;            werden nicht aufgefuellt - die Daten bleiben als Muell stehen 
; 
; Registerzuweisung:
;	
; (HL) Zeiger auf Lesebuffer
; (DE) Zeiger auf Schreibbuffer
; (C)  Bytezaehler
; (B)  Pruefsumme  
;
; Fehlercode:
;
; (A)=01h CY=1 kein ASII-Zeichen
; (A)=02h Z=1 Starmarke nicht gefunden
; (A)=02h Z=1 Pruefsumme falsch
; (A)=02h Z=1 Dateiende (EOF)
;
; Unterprogram: asc_h
;
; Startmarke ":" holen
;
hxi_:		xra	a		; Registervorbereiten (DE+BC)
		mov	b,a		;
		mov	c,a		;
		mov	d,a		;
		mov	e,a		;
		push	b		;
		mvi	c,0fh		; Fehlerzaehler setzen
hxi_sl:	mov	a,m		; Startmarke suchen
		cpi	3ah		;
		jz	hxi_ln	; weiter mit Laenge holen
					; Zeilenende(abspann) behandeln
		cpi	0Dh		; test auf CR
		jz	hxi_mt	;
		dcr	c		; Fehlerzaehler testen
		jz	hxi_erm 	; keine Startmarke gefunden
hxi_mt:	inx	h		; nachstes Byte holen
		jmp	hxi_sl	;

; Laenge holen

hxi_ln:	pop	b		;
		inx	h		; 2.Byte holen
		call	asc_h		; ASCII->HEX
		jc	hxi_era	; Fehler kein ASCII
		mov	c,a		; Bytezaehler laden
		add	b		;
		mov	b,a		; Pruefsumme schreiben
	
; Adresse holen

		inx	h		; HighByte
		call	asc_h		;
		jc	hxi_era	; Fehler kein ASCII
		mov	d,a		;
		add	b		;
		mov	b,a		; Pruefsumme schreiben
		inx	h		; LowByte
		call asc_h
		jc	hxi_era	; Fehler kein ASCII
		mov	e,a		; 
		add	b		;
		mov	b,a		; Pruefsumme schreiben

; Type holen

		inx	h
		call	asc_h
		jc	hxi_era	; Fehler kein ASCII
		cpi	00h		; Nutzdaten folgen
		jnz	hxi_erf	; EOF erreicht
		add	b		;
		mov	b,a		; Pruefsumme schreiben
		
; Daten holen

hxi_dl:	inx	h		;
		mov	a,m		;
		call	asc_h		;
		jc	hxi_era	; Fehler kein ASCII
		push	psw
		add	b		;
		mov	b,a		; Pruefsumme schreiben
		pop	psw
		xchg			;
		mov	m,a		; Datenbyte in Zielbuffer
		xchg			;
		inx	d		;
		dcr	c		;
		jnz	hxi_dl	; naechstes Byte

; Pruefsumme holen

		inx	h		;
		mov	a,m		;
		call	asc_h		;
		jc	hxi_era	; Fehler kein ASCII
		add	b		;
		cpi	0h		;
		jnz	hxi_ers	; Fehler Pruefsumme
		inx	h		;
hxi_end: 	ret 			
;
; Fehlerbehandlung
;
hxi_era: 	ret			; (A)=01h CY=1 kein ASII-Zeichen

hxi_erm: 	xra	a		; (A)=02h Z=1 Starmarke nicht gefunden
		mvi	a,02h
		ret			;
		
hxi_ers:	xra	a		; (A)=03h Z=1 Pruefsumme falsch
		mvi	a,03h
		ret			;
	
hxi_erf:	xra	a		; (A)=04h Z=1Dateiende (EOF)
		mvi	a,04h
		ret
;
;----------------------------------------------------
;
; ASCII -> HEX 
; Teile aus Profimonitor rot entnommen (C)Rheinhard Goessler hms
; 
; (HL) Zeiger fuer Quellbuffer
;  (A) Hexwert
; 
; Fehler: (A)=00h + CY=1 kein ASCII zeichen)
;
asc_h:	push	b		;
		push	d
		mov	a,m		; 1. zeichen holen
		call	hxtst
		jc	asc_err	; Fehler 
		rlc			; Nible in obere Haelfte
		rlc
		rlc
		rlc
		ani	0F0h		; unters Nible ausblenden
		mov	b,a		;
		inx	h		; 2. Zeichen holen
		mov	a,m
		call	hxtst
		jc	asc_err	; Fehler 
		ani	0Fh		; oberes Nible ausblenden
		ora	b		;
		
asc_err:	pop	d
		pop	b
		ret			; (A) Hex
; 
hxtst: 	push	b
		ani	5fh   	;Grossbuchstabe
       	cpi	10h   	;"0"
       	jc 	hexerr	;Code zu klein:JMP
       	cpi	1ah   	;"9"+1
       	jc 	hexcte	;Code o.k.:JMP
       	adi	0c9h  	;A...F adjust
       	cpi	10h   	;"F"+1
       	jnc	hexerr	;Code zu gross:JMP
       	cpi	0ah   	;korr. "A"
       	jnc	insft 	;A...F: JMP
	
hexerr:	xra	a		;FLAGs setzen
       	mvi	a,03h		;kein HEX-Digit
		rar
		pop	b
       	ret			;Fehler: CY=1
;
hexcte:	sui	10h		;Rest der "3x"
insft:	mov	b,a
		xra	a		;Flags lschen
		mov	a,b
		pop	b
		ret   	      ;REG A: HEX-Byte
;
;--------------------------------------------------	
; Texte fr Bildschirmmeldungen
;
meld_0:	.db	0dh		; CR  Startmeldung
		.db	0ah		; LF
		.db	"Intel-Hex "
		.db	"Ladeprogramm"
		.db	" V3.5 (C)" 
		.db	"W.Roemer 06.16"
		.db	00h		; String-Ende
			
meld_1:	.db	0dh		; 
		.db	0ah		; 
		.db	"Sender starten"
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		;  
			
meld_2:	.db	0dh		; 
		.db	0ah		; 
		.db	"Empfang Ende"
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		; 
			
meld_3:	.db	0dh		; 
		.db	0ah		; 
		.db	"Empfaenger"
		.db	" starten"	
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		; 
			
meld_4:	.db	0dh		; 
		.db	0ah		; 
		.db	"Uebertragung"
		.db	" beendet"
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		; 
		
meld_5:	.db	0dh		; 
		.db	0ah		; 
		.db	"Hex-Interpreter"
		.db	" fertig"
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		; 
;----------------------------------------------------		
; Fehlermeldungen
;
err_0:	.db	0dh		;
		.db	0ah		;
		.db	" Uebertragungs"
		.db	"fehler"
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		;
			
err_1:	.db	0dh		; 
		.db	0ah		; 
		.db	" kein ASCII"
		.db	" Zeichen"
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		;
		
err_2:	.db	0dh		; 
		.db	0ah		; 
		.db	" kein Startmarke"
		.db	" gefunden"
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		;
	
err_3:	.db	0dh		; 
		.db	0ah		; 
		.db	" Pruefsumme"
		.db	" falsch"
		.db	0dh		; 
		.db	0ah		; 
		.db	00h		;	
		
errmsg:	.db	0Dh		; 
		.db	0Ah		;    
		.db	"Error F0"
		.db	0Dh		;
		.db	00h	 
			
errmsg1:	.db	0Dh		; 
		.db	0Ah		;  
		.db	" CF-Karte "
		.db	"nicht vorhanden"
		.db	0Dh		;
		.db	00h	 
		
;	
;====================================================================
;
;        
.end	


